@@ -1,5 +1,7 @@
 import numpy as np
+
 from ipywidgets import widgets, Layout
+
 from tqdm.notebook import tqdm as tqdm_notebook
 
 
@@ -9,7 +11,8 @@ class ProgressBar(tqdm_notebook):
         # self.container.children[0].layout = Layout(width="80%")
 
 
-def make_trial_event_controller(trials, layout=None):
+def make_trial_event_controller(trials, layout=None, multiple=False):
+    """Controller for which reference to use (e.g. start_time) when making time-aligned averages"""
     trial_events = ["start_time"]
     if not np.all(np.isnan(trials["stop_time"].data)):
         trial_events.append("stop_time")
@@ -21,7 +24,20 @@ def make_trial_event_controller(trials, layout=None):
     kwargs = {}
     if layout is not None:
         kwargs.update(layout=layout)
-    trial_event_controller = widgets.Dropdown(
-        options=trial_events, value="start_time", description="align to: ", **kwargs
-    )
+
+    if multiple:
+        trial_event_controller = widgets.SelectMultiple(
+            options=trial_events,
+            value=["start_time"],
+            description='align to:',
+            disabled=False,
+            **kwargs
+        )
+    else:
+        trial_event_controller = widgets.Dropdown(
+            options=trial_events,
+            value="start_time",
+            description="align to: ",
+            **kwargs
+        )
     return trial_event_controller
