@@ -2,13 +2,17 @@ from collections import OrderedDict
 
 import h5py
 import hdmf
-import ndx_grayscalevolume
 import pynwb
-import zarr
+
 from ipywidgets import widgets
+
+import zarr
+import ndx_grayscalevolume
 from ndx_icephys_meta.icephys import SweepSequences
 from ndx_spectrum import Spectrum
 
+from .dynamictablesummary import DynamicTableSummaryWidget
+
 from nwbwidgets import (
     behavior,
     misc,
@@ -34,12 +38,14 @@ default_neurodata_vis_spec = {
     pynwb.NWBFile: file.show_nwbfile,
     SweepSequences: icephys.show_sweep_sequences,
     pynwb.behavior.BehavioralEvents: behavior.show_behavioral_events,
-    pynwb.ecephys.LFP: ecephys.show_lfp,
     pynwb.misc.Units: OrderedDict(
         {
+            "Summary": DynamicTableSummaryWidget,
             "Session Raster": misc.RasterWidget,
             "Grouped PSTH": misc.PSTHWidget,
             "Raster Grid": misc.RasterGridWidget,
+            "Tuning Curves": misc.TuningCurveWidget,
+            "Combined": misc.TuningCurveExtendedWidget,
             "table": show_dynamic_table,
         }
     ),
@@ -53,19 +59,20 @@ default_neurodata_vis_spec = {
     pynwb.ophys.DfOverF: ophys.show_df_over_f,
     pynwb.ophys.RoiResponseSeries: ophys.RoiResponseSeriesWidget,
     pynwb.misc.AnnotationSeries: OrderedDict(
-        {"text": base.show_text_fields, "times": misc.show_annotations}
+        {
+            "text": base.show_text_fields,
+            "times": misc.show_annotations
+        }
     ),
     pynwb.core.LabelledDict: base.dict2accordion,
     pynwb.ProcessingModule: base.processing_module,
-    hdmf.common.DynamicTable: show_dynamic_table,
-    pynwb.ecephys.ElectricalSeries: ecephys.ElectricalSeriesWidget,
-    pynwb.behavior.Position: behavior.show_position,
-    pynwb.behavior.SpatialSeries: OrderedDict(
+    hdmf.common.DynamicTable:
         {
-            "over time": timeseries.SeparateTracesPlotlyWidget,
-            "trace": behavior.plotly_show_spatial_trace,
-        }
-    ),
+        "Summary": DynamicTableSummaryWidget,
+        "table": show_dynamic_table,
+        },
+    pynwb.ecephys.ElectricalSeries: ecephys.ElectricalSeriesWidget,
+    pynwb.behavior.SpatialSeries: behavior.route_spatial_series,
     pynwb.image.GrayscaleImage: image.show_grayscale_image,
     pynwb.image.RGBImage: image.show_rbga_image,
     pynwb.image.RGBAImage: image.show_rbga_image,
@@ -73,12 +80,17 @@ default_neurodata_vis_spec = {
     pynwb.image.ImageSeries: image.ImageSeriesWidget,
     pynwb.image.IndexSeries: image.show_index_series,
     pynwb.TimeSeries: timeseries.show_timeseries,
+    pynwb.core.MultiContainerInterface: base.show_multi_container_interface,
     pynwb.core.NWBContainer: base.show_neurodata_base,
     pynwb.core.NWBDataInterface: base.show_neurodata_base,
     h5py.Dataset: base.show_dset,
     zarr.core.Array: base.show_dset,
     Spectrum: spectrum.show_spectrum,
-    pynwb.behavior.CompassDirection: behavior.show_position,
+    pynwb.icephys.SequentialRecordingsTable: {
+        "Summary": DynamicTableSummaryWidget,
+        "table": show_dynamic_table,
+        "I-V Analysis": icephys.IVCurveWidget
+    }
 }
 
 
